//----------------------------------------------------
//
// File:   utilpanl.c
// Author: Patrice BERNARD (metro@nanika.net)
//
// Preference panel management
//
//----------------------------------------------------
//
// Copyright (c) Patrice BERNARD 2005
//
//----------------------------------------------------
//
// Version 1.0.0 (12/03/2005)
// - Creation
//
//----------------------------------------------------

#include <PalmOS.h>

#include "resources.rh"

//----------------------------------------------------
// Global variables
//----------------------------------------------------
// Different UI if called from app ("done" button and no selection list)
static Boolean             CalledFromAppMode;

// What's required to list the panels
static UInt16              panelCount;
static MemHandle           panelIDsH = 0;
static SysDBListItemType * panelIDsP = NULL;

//----------------------------------------------------
// Local functions
//----------------------------------------------------
static void PanelCreatePickList(ControlPtr triggerP, ListPtr listP, ListDrawDataFuncPtr func);
static void PanelPickListDrawItem(Int16 itemNum, RectanglePtr bounds, Char ** data);

//----------------------------------------------------
// PanelSetCallMode()
// Register call mode (if called from app)
//----------------------------------------------------
void PanelSetCallMode(Boolean calledFromAppMode, UInt32 crid)
{
	PrefActivePanelParamsType prefs;

	// Save mode
	CalledFromAppMode = calledFromAppMode;

	if (!CalledFromAppMode)
	{
		// Set this panel as the active one next time the prefs app is opened
		prefs.activePanel = crid;
		AppCallWithCommand(sysFileCPreferences, prefAppLaunchCmdSetActivePanel, &prefs);
	}
}

//----------------------------------------------------
// PanelFormInit()
// Initialize UI (depending on calling mode)
//----------------------------------------------------
void PanelFormInit(FormPtr form)
{
	// Configure the form differently if called from app
	if (CalledFromAppMode)
	{
		// Replace selection list by label
		FrmHideObject(form, FrmGetObjectIndex(form, ID_TRIG_PANL));
		FrmShowObject(form, FrmGetObjectIndex(form, ID_LABL_TITL));

		// Show "done" button
		FrmShowObject(form, FrmGetObjectIndex(form, ID_BUTN_CLOS));
	}
	// Create the panel selection list
	else
	{
		PanelCreatePickList(FrmGetObjectPtr(form, FrmGetObjectIndex(form, ID_TRIG_PANL)), FrmGetObjectPtr(form, FrmGetObjectIndex(form, ID_LIST_PANL)), PanelPickListDrawItem);
	}
}

//----------------------------------------------------
// PanelFormClose()
// Clean-up after form is closed
//----------------------------------------------------
void PanelFormClose(FormPtr form)
{
	// Free memory
	if (panelIDsH != 0)
	{
		if (panelCount > 0)
		{
			MemHandleUnlock(panelIDsH);
		}
		MemHandleFree(panelIDsH);
	}
}

//----------------------------------------------------
// PanelSelectPanel()
// Selection in the panel list
//----------------------------------------------------
Boolean PanelSelectPanel(EventType * event)
{
	UInt16  card;
	LocalID dbid;
	Int16   select;

	// Selection in panel list
	if (event->data.popSelect.listID == ID_LIST_PANL)
	{
		// Get new selection
		select = event->data.popSelect.selection;

		// Get current panel
		SysCurAppDatabase(&card, &dbid);

		// Check if switch to a different one
		if (panelIDsP[select].dbID != dbid
		 || panelIDsP[select].cardNo != card)
		{
			// Switch app
			SysUIAppSwitch(panelIDsP[select].cardNo, panelIDsP[select].dbID, sysAppLaunchCmdNormalLaunch, 0);
		}

		// Processed
		return true;
	}

	return false;
}

//----------------------------------------------------
// PanelSelectClose()
// Close button action processing
//----------------------------------------------------
Boolean PanelSelectClose(EventType * event)
{
	EventType newEvent;

	// Close button?
	if (event->data.ctlSelect.controlID == ID_BUTN_CLOS)
	{
		// Post a quit event (return to caller)
		newEvent.eType = appStopEvent;
		EvtAddEventToQueue(&newEvent);

		// Processed
		return true;
	}

	return false;
}

//----------------------------------------------------
// PanelCreatePickList()
// Build the list of panels
//----------------------------------------------------
static void PanelCreatePickList(ControlPtr triggerP, ListPtr listP, ListDrawDataFuncPtr func)
{
	UInt16  card;
	LocalID dbid;
	int item;

	SysCreatePanelList(&panelCount, &panelIDsH);

	if (panelCount > 0)
	{
		panelIDsP = MemHandleLock(panelIDsH);
	}

	// Set GUI list content
	LstSetListChoices(listP, NULL, panelCount);

	// Now resize the list to the number of panel found
	LstSetHeight(listP, panelCount);

	// Set drawing function
	LstSetDrawFunction(listP, func);

	// Select current panel in list
	SysCurAppDatabase(&card, &dbid);
	for (item = 0; item < panelCount; item++)
	{
		if (panelIDsP[item].dbID == dbid && panelIDsP[item].cardNo == card)
		{
			 LstSetSelection(listP, item);
			 CtlSetLabel(triggerP, panelIDsP[item].name);
			 break;
		}
	}
}

//----------------------------------------------------
// PanelPickListDrawItem()
// Draw one item in the panel list
//----------------------------------------------------
static void PanelPickListDrawItem(Int16 itemNum, RectanglePtr bounds, Char ** data)
{
	Char * itemText = panelIDsP[itemNum].name;
	WinDrawChars(itemText, StrLen(itemText), bounds->topLeft.x, bounds->topLeft.y);
}
